pub mod asv;
pub mod pytest;

use crate::{Adaptable, AdapterResults, Settings};
use asv::AdapterPythonAsv;
use pytest::AdapterPythonPytest;

pub struct AdapterPython;

impl Adaptable for AdapterPython {
    fn parse(input: &str, settings: Settings) -> Option<AdapterResults> {
        AdapterPythonAsv::parse(input, settings)
            .or_else(|| AdapterPythonPytest::parse(input, settings))
    }
}

#[cfg(test)]
mod test_go {
    use super::AdapterPython;
    use crate::adapters::{
        python::asv::test_python_asv, python::pytest::test_python_pytest,
        test_util::convert_file_path,
    };

    #[test]
    fn test_adapter_python_asv() {
        let results = convert_file_path::<AdapterPython>("./tool_output/python/asv/six.txt");
        test_python_asv::validate_adapter_python_asv(&results);
    }

    #[test]
    fn test_adapter_python_pytest() {
        let results = convert_file_path::<AdapterPython>("./tool_output/python/pytest/four.json");
        test_python_pytest::validate_adapter_python_pytest(&results);
    }
}
